//*****************************************************************************
//
// Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//  Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  Redistributions in binary form must reproduce the above copyright
//  notice, this list of conditions and the following disclaimer in the
//  documentation and/or other materials provided with the
//  distribution.
//
//  Neither the name of Texas Instruments Incorporated nor the names of
//  its contributors may be used to endorse or promote products derived
//  from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//****************************************************************************
#include <stdbool.h>
#include <stdint.h>
#include "msp432.h"
#include "i2c_driver.h"
#include "driverlib.h"

//*****************************************************************************
//
// Definitions
//
//*****************************************************************************
#define EUSCI_I2C_STATUS_SLAVE_NACK       1

/* EUSCI port and pin selection */
#define EUSCI_I2C_MODULE                  EUSCI_B1_BASE
#define EUSCI_I2C_PORT                    GPIO_PORT_P6
#define EUSCI_I2C_SCL_PIN                 GPIO_PIN5
#define EUSCI_I2C_SCL_PIN_FUNCTION        GPIO_PRIMARY_MODULE_FUNCTION
#define EUSCI_I2C_SDA_PIN                 GPIO_PIN4
#define EUSCI_I2C_SDA_PIN_FUNCTION        GPIO_PRIMARY_MODULE_FUNCTION

//*****************************************************************************
//
// Global Data
//
//*****************************************************************************
/* I2C Master Configuration Parameter */
volatile eUSCI_I2C_MasterConfig i2cConfig =
{
        EUSCI_B_I2C_CLOCKSOURCE_SMCLK,          // SMCLK Clock Source
		0,
		EUSCI_B_I2C_SET_DATA_RATE_400KBPS,      // Desired I2C Clock of 400khz
        0,                                      // No byte counter threshold
		EUSCI_B_I2C_NO_AUTO_STOP
};

//*****************************************************************************
//
// Imported Data
//
//*****************************************************************************

//*****************************************************************************
//
// Constants
//
//*****************************************************************************

//*****************************************************************************
//
// Function Prototypes
//
//*****************************************************************************

/***********************************************************
  Function:
*/
void initI2C(void)
{
	/* I2C Clock Soruce Speed */
	i2cConfig.i2cClk = MAP_CS_getSMCLK();

    /* Select I2C function for I2C_SCL & I2C_SDA */
    GPIO_setAsPeripheralModuleFunctionOutputPin(EUSCI_I2C_PORT, EUSCI_I2C_SCL_PIN,
    		EUSCI_I2C_SCL_PIN_FUNCTION);
    GPIO_setAsPeripheralModuleFunctionOutputPin(EUSCI_I2C_PORT, EUSCI_I2C_SDA_PIN,
    		EUSCI_I2C_SDA_PIN_FUNCTION);

    /* Initializing I2C Master to SMCLK at 400kbs with no-autostop */
    MAP_I2C_initMaster(EUSCI_I2C_MODULE, (const eUSCI_I2C_MasterConfig *)&i2cConfig);
	
    /* Enable I2C Module to start operations */
	MAP_I2C_enableModule(EUSCI_I2C_MODULE);
}

/***********************************************************
  Function:
*/
bool writeI2C(uint8_t ui8Addr, uint8_t ui8Reg, uint8_t *Data, uint8_t ui8ByteCount)
{
	/* Todo: Implement a delay */
	/* Wait until ready to write */
    while (MAP_I2C_isBusBusy(EUSCI_I2C_MODULE));

	/* Load device slave address */
	MAP_I2C_setSlaveAddress(EUSCI_I2C_MODULE, ui8Addr);

	/* Send start bit and register */
  	MAP_I2C_masterSendMultiByteStart(EUSCI_I2C_MODULE,ui8Reg);

  	/* Wait for tx to complete */
	while(!(MAP_I2C_getInterruptStatus(EUSCI_I2C_MODULE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0) &
			EUSCI_B_I2C_TRANSMIT_INTERRUPT0));
	
    /* Check if slave ACK/NACK */
	if((MAP_I2C_getInterruptStatus(EUSCI_I2C_MODULE, EUSCI_B_I2C_NAK_INTERRUPT)) &
			EUSCI_B_I2C_NAK_INTERRUPT)
    {
        /* If NACK, set stop bit and exit */
		MAP_I2C_masterSendMultiByteStop(EUSCI_I2C_MODULE);
        return(EUSCI_I2C_STATUS_SLAVE_NACK);
    }

	/* Now write one or more data bytes */
    while(1)
    {
        /* Wait for next INT */
    	while(!(MAP_I2C_getInterruptStatus(EUSCI_I2C_MODULE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0) &
    			EUSCI_B_I2C_TRANSMIT_INTERRUPT0));

        /* If no data to follow, we are done */
        if(ui8ByteCount == 0 )
        {
			MAP_I2C_masterSendMultiByteStop(EUSCI_I2C_MODULE);
			MAP_I2C_clearInterruptFlag(EUSCI_I2C_MODULE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0);
            return(true);
        }
        /* If more, send the next byte */
        else
        {
			MAP_I2C_masterSendMultiByteNext(EUSCI_I2C_MODULE, *Data++);
        }
        ui8ByteCount--;
    }
}

/***********************************************************
  Function:
*/
bool readI2C(uint8_t ui8Addr, uint8_t ui8Reg, uint8_t *Data, uint8_t ui8ByteCount)
{
	/* Todo: Implement a delay */
	/* Wait until ready */
    while (MAP_I2C_isBusBusy(EUSCI_I2C_MODULE));

	/* Load device slave address */
	MAP_I2C_setSlaveAddress(EUSCI_I2C_MODULE, ui8Addr);

  	/* Send start bit and register */
  	MAP_I2C_masterSendMultiByteStart(EUSCI_I2C_MODULE,ui8Reg);

  	/* Wait for tx to complete */
	while(!(MAP_I2C_getInterruptStatus(EUSCI_I2C_MODULE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0) &
			EUSCI_B_I2C_TRANSMIT_INTERRUPT0));

    /* Check if slave ACK/NACK */
	if((MAP_I2C_getInterruptStatus(EUSCI_I2C_MODULE, EUSCI_B_I2C_NAK_INTERRUPT)) &
			EUSCI_B_I2C_NAK_INTERRUPT)
    {
        /* If NACK, set stop bit and exit */
		MAP_I2C_masterSendMultiByteStop(EUSCI_I2C_MODULE);
        return(EUSCI_I2C_STATUS_SLAVE_NACK);
    }

	/* Turn off TX and generate RE-Start */
  	MAP_I2C_masterReceiveStart(EUSCI_I2C_MODULE);

  	/* Wait for start bit to complete */
    while(MAP_I2C_masterIsStartSent(EUSCI_I2C_MODULE));

	if((MAP_I2C_getInterruptStatus(EUSCI_I2C_MODULE, EUSCI_B_I2C_NAK_INTERRUPT)) &
			EUSCI_B_I2C_NAK_INTERRUPT)
    {
        /* If NACK, set stop bit and exit */
		MAP_I2C_masterSendMultiByteStop(EUSCI_I2C_MODULE);
        return(EUSCI_I2C_STATUS_SLAVE_NACK);
    }
			
    /* Read one or more bytes */
    while(ui8ByteCount)
    {
        /* If reading 1 byte (or last byte), generate the stop to meet the spec */
        if(ui8ByteCount-- == 1)
        {
        	*Data++ = MAP_I2C_masterReceiveMultiByteFinish(EUSCI_I2C_MODULE);
        }
        else
        {
        	/* Wait for next RX interrupt */
        	while(!(MAP_I2C_getInterruptStatus(EUSCI_I2C_MODULE, EUSCI_B_I2C_RECEIVE_INTERRUPT0) &
        			EUSCI_B_I2C_RECEIVE_INTERRUPT0));

        	/* Read the rx byte */
        	*Data++ = MAP_I2C_masterReceiveMultiByteNext(EUSCI_I2C_MODULE);
        }
    }

	MAP_I2C_clearInterruptFlag(EUSCI_I2C_MODULE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0);
    return(true);
}

/***********************************************************
  Function: euscib1IntHandler
 */
void euscib1IntHandler(void)
{
    uint_fast16_t status;

    status = MAP_I2C_getEnabledInterruptStatus(EUSCI_I2C_MODULE);
    MAP_I2C_clearInterruptFlag(EUSCI_I2C_MODULE, status);

    if (status & EUSCI_B_I2C_NAK_INTERRUPT)
    {
    }

    if (status & EUSCI_B_I2C_START_INTERRUPT)
    {
    }

    if (status & EUSCI_B_I2C_STOP_INTERRUPT)
    {
    }

    if (status & EUSCI_B_I2C_RECEIVE_INTERRUPT0)
    {
    }

    if (status & EUSCI_B_I2C_TRANSMIT_INTERRUPT0)
    {
    }
}

